/*! **************************************************************************
 *	 Copyright (C) Siemens AG 2010 All Rights Reserved. Confidential
 *****************************************************************************
 *	 Written by SIEMENS - I MO	Confidential
 *****************************************************************************
 *
 *	 Projekt: SFC-Umrichter
 *
 *****************************************************************************
\file
\class KMPM
\ingroup pkgCCP_MES
\brief Proxy Messsystem

Dieser Baustein dient als Proxy zum Messsystem. \n Er speichert und verarbeitet alle von der MPM empfangenen Signale und fhrt, im Zusammenspiel mit anderen Bausteinen, z.B. \ref KSEQ "zentrale Steuerung", diverse Prfungen der 3~ und 1~ Netzgrssen Spannung, Strom und Frequenz durch.

<b >Symbol </b> \n
\image html	FB-KMPM.jpg[KMPM]

<b>Arbeitsweise</b>\n
3~ netzseitige Funktionen:
- \ref KMPM_checkSynchA "Prfung der 3~ Synchronitt"
- \ref KMPM_checkFA "berwachung der Netzfrequenz"
- \ref KMPM_checkUAF_Overvoltage "berwachung auf berspannung"
- \ref KMPM_checkUAF_Undervoltage "berwachung auf Unterspannung"
- \ref KMPM_checkUAF_1PhaseEarth "berwachung auf einphasiger Erdschluss"
- \ref KMPM_checkUAF_Fuse "berwachung auf Sicherungsfall im Strang"

1~ bahnseitige Funktionen:
- \ref KMPM_checkSynchB "Prfung der 1~ Synchronitt"
- \ref KMPM_checkUB "berwachung der Bahnspannung"
- \ref KMPM_checkFB "berwachung der Bahnfrequenz"
- \ref KMPM_checkIB "berwachung des Bahnstroms"

<b>Initialisierung</b>\n
Der Baustein allociert im Anlauf Speicher fr die eigene Struktur ZMPM_IO
und beschreibt die interne Struktur	YSfcModule zur Boardanbindung. 

<b>Besonderheiten</b>\n
\attention Die Funktion der MPM ist auf die beiden Prozessoren aufgeteilt.
Unterschiede auf den Prozessoren:


 
Methoden: \n
 - \ref KMPM_INI 
 - \ref KMPM_NRM 
 - \ref KMPM_initTimer
 - \ref KMPM_checkUA
 - \ref KMPM_checkUAF_Overvoltage
 - \ref KMPM_checkUAF_Undervoltage
 - \ref KMPM_calcVarTimeForUndervoltage
 - \ref KMPM_checkUAF_1PhaseEarth
 - \ref KMPM_checkUAF_Fuse
 - \ref KMPM_checkFA
 - \ref KMPM_checkSynchA
 - \ref KMPM_checkUB
 - \ref KMPM_checkUBF_Overvoltage
 - \ref KMPM_checkUBF_Undervoltage
 - \ref KMPM_checkIB
 - \ref KMPM_checkFB
 - \ref KMPM_checkSynchB
 - \ref KMPM_calcEfficiency
 
 
@dontinclude KMPM.dat
\b Projektierungsdaten
@skipline ALARM
@skipline ONLINE_INSERT
@skipline INI_MODE
@skipline RUNTIME
 
\b Anschlsse \n
- \b Eingangskonnektoren \n
@skipline VAR_INPUT
@skipline COMMENT
@until END_VAR
 
- \b Ausgangskonnektoren \n
@skipline VAR_OUTPUT
@skipline COMMENT
@until END_VAR
	
*/

FBTNAM(KMPM)

#include "p32.h"
#include "D7EXT.h"
#include "ANLAGE.h"
#include "math.h"
#include "kmpm.h"


/* FBAI */
/*! \brief INI ()
 * \anchor KMPM_INI \n
 *	*/
INI () {
	MAC_C_ALE(kmpm_ale);
	struct ZMPM_IO	*pOWN;
	pOWN = (ZMPM_IO*) malloc(sizeof(ZMPM_IO));

	YEV=0x00;
	if (pOWN == NIL) {
		 YEV |= kFB_noMemory;
	} else {
	//! Vorbereitung SfcModule zum Eintrag ins \ref KBOARD "SFC-Board"
		FB_initSFC_Module(&pOWN->cfgBoard, kId_MPM);

		pOWN->cFaults.mno1 = kMSG_MPM_firstMsg;
		pOWN->cFaults.mno2 = kMSG_MPM_secondMsg;
		KMPM_initFaultCategories(pOWN);

		pOWN->Tabtast = ta_info_msec();
		PFW = (uint32)&pOWN->cFaults;

		DXA = (int32)&pOWN->XA;
		pOWN->msgPLL_A.mno = kMSG_PLL_A;
		SPA = (uint32)&pOWN->msgPLL_A;
		pOWN->statusUAF.ib.eUNallocated = 1;

		pOWN->faultUAN.mno = kMSG_MPM_UAN;
		PAN = (int32)&pOWN->faultUAN; 
		pOWN->faultUAF.mno = kMSG_MPM_UAF;
		PAF = (int32)&pOWN->faultUAF; 
		pOWN->faultIA.mno  = kMSG_MPM_IA ;
		PIA = (int32)&pOWN->faultIA ; 
		pOWN->faultFA.mno  = kMSG_MPM_FA ;
		PFA = (int32)&pOWN->faultFA ; 

		DXB = (int32)&pOWN->XB;
		pOWN->msgPLL_B.mno = kMSG_PLL_B;
		SPB = (uint32)&pOWN->msgPLL_B;
		pOWN->statusUBF.ib.eUNallocated = 1;
		pOWN->faultUBN.mno = kMSG_MPM_UBN;
		pOWN->faultUBF.mno = kMSG_MPM_UBF;
		pOWN->faultIB.mno  = kMSG_MPM_IB ;
		pOWN->faultFB.mno  = kMSG_MPM_FB ;
		PBN = (int32)&pOWN->faultUBN; 
		PBF = (int32)&pOWN->faultUBF; 
		PIB = (int32)&pOWN->faultIB ; 
		PFB = (int32)&pOWN->faultFB ; 

		pOWN->iAK.akBase.akType = kAK_MPM;

		switch(PRZ){
			case 1:
				PSP = (uint32)&pOWN->ipc1;
				PRP = (uint32)&pOWN->ipc2;
			break;
			case 2:
				PRP = (uint32)&pOWN->ipc1;
				PSP = (uint32)&pOWN->ipc2;
			break;
		}

		POS = (uint32)pOWN;
		XAK = (uint32)&pOWN->iAK;

		// Initialisierung
		*(uint16*)&pOWN->statusA = 0x0;
		*(uint16*)&pOWN->statusB = 0x0;
	}
	if (YEV != 0) {
		SET_B1_LOG1(QF);	//memory error
	} else {
		SET_B1_LOG0(QF);
	}

	MAN = kMSG_MPM_UAN;
	MAF = kMSG_MPM_UAF;
	MIA = kMSG_MPM_IA ;
	MFA = kMSG_MPM_FA ;
	MCA = kMSG_MPM_AC;

	MBN = kMSG_MPM_UBN;
	MBF = kMSG_MPM_UBF;
	MIB = kMSG_MPM_IB ;
	MFB = kMSG_MPM_FB ;
	MCB = kMSG_MPM_BC;

	LEN = sizeof(ZMPM_IO);
	TA	= ta_info_msec();
}
/* FBA0 */
/*! \brief NRM ()
 * \anchor KMPM_NRM \n
 * @~german
zyklischer Programmanteil

@~english
cyclic program */
NRM () {
	MAC_C_ALE(kmpm_ale);
	struct ZMPM_IO	*pOWN;

	YSFC_Board*	pBoard;
	// Anlagenzeiger
	struct ZAK	    *pAK;
	struct ZA3P_IO  *pA3P;
	struct ZCCS_IO  *pCCS;
	struct ZREF_IO  *pREF;
	struct ZSEQ_IO  *pSEQ;
	struct ZSEQ2_IO *pSEQ2;
	YSFCTimer	*pTimer;
	bool		 fTimeout;

	eNET_TYPE	 netType;
	float rama;
	int it;
	bool releaseFueb;
	bool localForceAsyn;
	struct YMPM_XA* pXA;
	struct YMPM_XB* pXB;

	if(IS_B1_LOG1(QF)) return; //Error in memory-allocation

	pOWN	= (struct ZMPM_IO*)POS;
	pOWN->cfgBoard.moduleFaults = 0x0;
	pOWN->faults.iw = 0x0;
	//! Anbindung an das \ref KBOARD "SFC-Board"
	if(FB_checkBoardConnection(&pOWN->cfgBoard)) { 
		pBoard = (YSFC_Board*) pOWN->cfgBoard.pBoard;
		//! - Zeiger auf Partnermodule vom Board lesen	
		pAK	 = (ZAK*)		pBoard->Module[kId_KAK];
		pA3P = (ZA3P_IO*)	pBoard->Module[kId_A3P];
		pCCS = (ZCCS_IO*)	pBoard->Module[kId_CCS];
		pREF = (ZREF_IO*)	pBoard->Module[kId_REF];
		pSEQ = (ZSEQ_IO*)	pBoard->Module[kId_SEQ];
		pSEQ2= (ZSEQ2_IO*)	pBoard->Module[kId_SEQ2];
	} else {
		YEV = pOWN->cfgBoard.moduleFaults;
		return;
	}

	/*! - \ref YAK_MPM "Anlagenkenngrssen" aus \ref ZAK "Parameterdienst" bernehmen*/
	if( (pAK==NIL) || (pAK->valid != kAK_LOADED) )
	{	//inactive or no valid data
		pOWN->cfgBoard.moduleFaults = kFB_pAK;
		YEV = pOWN->cfgBoard.moduleFaults;
		return;
	}
	else if(pOWN->iAK.akBase.akState != kAK_LOADED)
	{
		memcpy(&pOWN->iAK ,(void*)&(pAK->akMPM) ,sizeof(YAK_MPM));
		pOWN->iAK.akBase.akState = kAK_LOADED;
		// Tabelleninhalt prfen
		if((pOWN->iAK.akA.akUN == 0.0)|| (pOWN->iAK.akB.akUN == 0.0))
		{
			pOWN->cfgBoard.moduleFaults = kFB_akValue;
			YEV = pOWN->cfgBoard.moduleFaults;
			return;
		}
		//! - \ref KMPM_initTimer "Parameter" zu den internen Zeitberwachungen bernehmen [ms]
		KMPM_initTimer(pOWN);
	}
	netType = NT;
	pOWN->wVarNet = PL2;
	localForceAsyn = (pREF->netType.forceASYN && pREF->netType.released) ? 1:0; //

	pOWN->prozNr = PRZ;
	switch(pOWN->prozNr)
	{
	// ###################################################################
	// #####							 Prozessor 1				 #####
	// ###################################################################
	//! <b>Funktionen auf Prozessor 1:</b>
		case (1):
		{
			if( (pA3P == NIL) ||
				(pCCS == NIL) ||
				(pSEQ == NIL) ||
				(pREF == NIL) )
			{
				YEV = kFB_pModuleMiss;
				return;
			}

		// ######################
		// ### Daten einlesen ###
		// ######################
			/*Die Eingangsdaten werden aus dem CFC ber den Eingangskonnektoren ICA + ICB eingelesen,
			 * zustzlich werden von den Peripheriebausteinen die binren IO in die Eingangsstruktur geroutet.
			 */
			KMPM_readPerCommands_A(pOWN, ICA);
			KMPM_readPerCommands_B(pOWN, ICB, pSEQ, pREF);

			// Anwahl UAN / UAF
			if(pOWN->Ein.cA_yard.ib.cSelectUmess == kUNselect) {
				pOWN->cA_yard.ib.cSelectUmess = kUNselect;
			}else{
				if((pOWN->Ein.cA_yard.ib.cSelectUmess == kUFselect) &&
				   (pCCS->ipc2.cFarSync.cSelectUAF) ){ // gespiegelte Vorgabe von P2
					pOWN->cA_yard.ib.cSelectUmess = kUFselect;
				}
			}

			//! - \ref YMPM_IPC2 "Daten aus Proz.2" bernehmen
			KMPM_readIPC2(pOWN);

			// Ausgabe auf Konnektoren zur Anzeige
			MWA = pOWN->mMPMA.dw;
			MWB = pOWN->mMPMB.dw;

			VB0 = pOWN->eVB0;
			VB1 = pOWN->eVB1;

		// ######################
		//!		3~ Netzseite
		// ######################
			if(pA3P->pCBa && pA3P->pCBc)
			{
				if((pA3P->pCBa->cbStatus.eClosed &&					// Normalumschaltung
				    pA3P->pCBc->cbStatus.eClosed &&
				    (pOWN->cA_yard.ib.cSelectUmess == kUFselect)&&
				    pCCS->ipc2.cFarSync.cSelectUAF ) )
				{
					pOWN->Ein.cA_yard.ib.cUsync_VLT = 0;
					pOWN->cA_yard.ib.cUsync_VLT = 0;
				}
			}
		// Ansteuerung der A-PLL
		KMPM_controlPLL_A(pOWN, pSEQ, pREF);

		// ######################
		//! 	 1~ Bahnseite	 
		// ######################
		// Ansteuerung der B-PLL
		KMPM_controlPLL_B(pOWN, pSEQ, pREF);

		// Berechnung des Lastfaktors mit PT1-Glttung
		if(pOWN->iAK.akB.akIN!=0.0)
		{
			pOWN->Tsmooth_LF = (TLF > 0.0) ? TLF : 1000.0;
			rama = (pOWN->Tabtast<pOWN->Tsmooth_LF) ? pOWN->Tabtast/pOWN->Tsmooth_LF : 1; // Begrenzung der dt-nderung, wenn Zyklus lnger als Restrampe
			pOWN->xLoadFactor += rama * ((pOWN->XB.xIBF / pOWN->iAK.akB.akIN) - pOWN->xLoadFactor);
		}
		else
		{
			pOWN->xLoadFactor = 0.0;
		}
		// xEfficiency mit PT1-Glttung
		pOWN->Tsmooth_EF = (TEF > 0.0) ? TEF : 1000.0;
		KMPM_calcEfficiency(pOWN);

		// Prfung, ob CCS-Notfallbehandlung (KS) aktiv
		fTimeout = 0;
		pTimer = &(pOWN->timer[kMpmTimer_UB_releaseSupervision]);
		if(!pCCS->ipc2.eCCS_CCP_MSG.mCB.e_B_KSno_YB) // KS-Notfallbehandlung -> min.Spannungsberwachung deaktivieren
		{
			SFC_TimerReset(pTimer);
		}else{
			if(pOWN->cB_yard.ib.eCBon && !pTimer->armed)
			{
				SFC_TimerLoad(pTimer);
			}
		}
		pOWN->cB_yard.ib.cSuperviseUBmin = SFC_TimerTimeout(pTimer);

		// Daten fr Proz.2 bereit stellen
		KMPM_prepareIPC1(pOWN);

		}break;

	// ###################################################################
	// #####							 Prozessor 2				 #####
	// ###################################################################
	//! <b>Funktionen auf Prozessor 2:</b>
		case (2):
		{
			if( (pCCS == NIL)||
				(pSEQ2== NIL))
			{
				pOWN->cfgBoard.moduleFaults = kFB_pModuleMiss;
				YEV = pOWN->cfgBoard.moduleFaults;
				return;
			}


		// ################################
		// ###		Daten einlesen		###
		// ################################
		//	pOWN->mMPM.MessageWord = MSW;
			pOWN->mMPMA.dw = MWA;
			pOWN->mMPMB.dw = MWB;
			pOWN->eVB0     = VB0;
			pOWN->eVB1     = VB1;

			// Daten aus Proz.1 bernehmen
			KMPM_readIPC1(pOWN);

		// ################################
		// ####		3~ Netzseite		####
		// ################################
			pXA = (struct YMPM_XA*) PXA;
			if(pXA != NIL)
			{
			//	pOWN->XA.eVB1 = pXA->eVB1;
			//	pOWN->XA.eVB2 = pXA->eVB2;
				pOWN->XA.eVB  = pXA->eVB;
				pOWN->XA.rNAM = pXA->rNAM;
				pOWN->XA.eMSW = pXA->eMSW;

				pOWN->XA.xUAF    = sqrtf(pXA->xUAF_mp*pXA->xUAF_mp + pXA->xUAF_mn*pXA->xUAF_mn);
				pOWN->XA.xUAF_mp = pXA->xUAF_mp ;
				pOWN->XA.xUAF_mn = pXA->xUAF_mn ;
				pOWN->XA.xUAF1   = pXA->xUAF1   ;
				pOWN->XA.xUAF2   = pXA->xUAF2   ;
				pOWN->XA.xUAF3   = pXA->xUAF3   ;

				pOWN->XA.xIAF    = sqrtf(pXA->xIAF_mp*pXA->xIAF_mp + pXA->xIAF_mn*pXA->xIAF_mn);
				pOWN->XA.xIAF_mp = pXA->xIAF_mp ;
				pOWN->XA.xIAF_mn = pXA->xIAF_mn ;
				pOWN->XA.xIAF1  = pXA->xIAF1    ;
				pOWN->XA.xIAF2  = pXA->xIAF2    ;
				pOWN->XA.xIAF3  = pXA->xIAF3    ;
				pOWN->XA.xPA    = pXA->xPA      ;
				pOWN->XA.xQA    = pXA->xQA      ;
				pOWN->XA.xSA    = sqrtf(pOWN->XA.xPA*pOWN->XA.xPA + pOWN->XA.xQA*pOWN->XA.xQA);
				if(pOWN->XA.xSA!=0.0){pOWN->XA.xCos_phi = pXA->xPA / pOWN->XA.xSA;}
				pOWN->XA.xUAN   = pXA->xUAN;
				pOWN->XA.xfA    = pXA->xfA;
			}
			// Anforderung der Messstelle, wenn notwendig
			pOWN->statusUAN.ib.eUNallocated = pOWN->cA_yard.ib.eUNallocated;
			// berprfung der Netzspannungen UAN + UAF
			KMPM_checkFA(pOWN);
			KMPM_checkUA(pOWN, pSEQ2);
			KMPM_checkUAF_1PhaseEarth(pOWN);
			KMPM_checkUAF_Fuse(pOWN, pSEQ2);
			pOWN->statusUAN.ib.eU_MCB_VT = pOWN->statusA.eUN_MCB_VT;
			pOWN->statusUAF.ib.eU_MCB_VT = pOWN->statusA.eUF_MCB_VT;

		// ################################
		// ####		1~ Bahnseite		####
		// ################################
			pXB = (struct YMPM_XB*) PXB;
			if(pXB != NIL)
			{
				// bernahme der Rohdaten und nachfolgende Berechnung einiger Zusatzwerte (OVERHEAD !!!)
				memcpy(&(pOWN->XB) ,(void*)pXB ,sizeof(YMPM_XB));

				pOWN->XB.xUBF    = pXB->xUBF_r;
				pOWN->XB.xIBF    = pXB->xIBF_r;
				pOWN->XB.xUBFF   = pXB->xUBF_m;
				pOWN->XB.xIBFF   = pXB->xIBF_m;
				pOWN->XB.xTHDibf = pXB->rNBF_THDI;

				pOWN->XB.xUBN    = pXB->xUBN;
				pOWN->XB.xUB     = pXB->xUBN;
				pOWN->XB.xfB     = pXB->xfB;
				pOWN->XB.xPB     = pXB->xPBF;
				pOWN->XB.xQB     = pXB->xQBF;
				pOWN->XB.xSB     = sqrtf(pXB->xPBF*pXB->xPBF + pXB->xQBF*pXB->xQBF);
				if(pOWN->XB.xSB != 0.0){pOWN->XB.xCos_phi = pOWN->XB.xPB / pOWN->XB.xSB;}
			}

			pOWN->statusUBN.ib.eUNallocated = pOWN->cB_yard.ib.eUNallocated;
			// berprfung der Bahnspannungen UBN + UBF, wenn keine Notfallbehandlung der CCS aktiv ist
			KMPM_checkUB(pOWN, pSEQ2, pCCS);
			KMPM_checkIB(pOWN, pCCS);

			// Abschaltung wg.Frequenzberwachung nur, wenn keine Notfallbehandlung der CCS aktiv ist
			releaseFueb = ((pCCS->ipc2.eCCS_CCP_MSG.mCB.e_B_KSno_YB)&&(pOWN->cB_yard.ib.cSuperviseFreq)) ? 1 : 0;
			KMPM_checkFB(pOWN,releaseFueb);
			pOWN->statusUBN.ib.eU_MCB_VT = pOWN->statusB.eUN_MCB_VT;
			pOWN->statusUBF.ib.eU_MCB_VT = pOWN->statusB.eUF_MCB_VT;

			// Daten fr Proz.1 bereit stellen
			KMPM_prepareIPC2(pOWN);
		}break;
		default:
		break;
	}
	pOWN->faults.ib.eShutOFFmin_UA = pOWN->statusUAF.ib.uShutOFFmin;
	pOWN->faults.ib.eShutOFFmax_UA = pOWN->statusUAF.ib.uShutOFFmax;
	pOWN->faults.ib.eShutOFFmin_UB = pOWN->statusUBF.ib.uShutOFFmin;
	pOWN->faults.ib.eShutOFFmax_UB = pOWN->statusUBF.ib.uShutOFFmax;
	pOWN->cFaults.faults = pOWN->faults.iw;
	YEV = pOWN->cfgBoard.moduleFaults;

	OCA = pOWN->cA_yard.iw[0];
	STA = *(uint16*)&pOWN->statusA;
	SAN = pOWN->statusUAN.iw;
	SAF = pOWN->statusUAF.iw;
	SIA = *(uint16*)&pOWN->statusIA.iw;
	SFA = *(uint16*)&pOWN->statusFA;
	pOWN->faultUAN.faults = pOWN->statusUAN.iw;
	pOWN->xFaultsUAN.iw    = pOWN->faultUAN.actFaults;

	pOWN->faultUAF.faults = pOWN->statusUAF.iw;
	pOWN->xFaultsUAF.iw    = pOWN->faultUAF.actFaults;

	pOWN->faultIA.faults  = *(uint16*)&pOWN->statusIA.iw;
	pOWN->faultFA.faults  = *(uint16*)&pOWN->statusFA;

	OCB = pOWN->cB_yard.iw[0];
	STB = *(uint16*)&pOWN->statusB;
	SBN = pOWN->statusUBN.iw;
	SBF = pOWN->statusUBF.iw;
	SIB = *(uint16*)&pOWN->statusIB.iw;
	SFB = *(uint16*)&pOWN->statusFB.iw;
	pOWN->faultUBN.faults = pOWN->statusUBN.iw;
	pOWN->faultUBF.faults = pOWN->statusUBF.iw;
	pOWN->faultIB.faults  = *(uint16*)&pOWN->statusIB.iw;
	pOWN->faultFB.faults  = *(uint16*)&pOWN->statusFB.iw;

	IB9 = pOWN->xI_B9;

	// Auswahl zur Anzeige der Zeitstufen
	it = IT;
	if(it < kMpmTimer_last){
		TIM = (uint32)&pOWN->timer[it];
	}

	IFF = pOWN->XB.xIBFF;
	IUE = pOWN->iAK.akB.akIUE;

	IEA = *(uint16*)&pOWN->Ein.cA_yard;
	IEB = *(uint16*)&pOWN->Ein.cB_yard;

	TST = pOWN->test;
	TF = pOWN->ytest.value[T];
	TI = pOWN->ytest.status[T];
}

/*! \brief KMPM_initTimer
 * \anchor KMPM_initTimer \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 * <b>Initialisierung der internen Zeitstufen</b>
 */
static void KMPM_initTimer(ZMPM_IO* pOWN) {
	int i;
	for(i=0;i<kMpmTimer_last;i++){
		SFC_TimerInit(&pOWN->timer[i]); //All timeouts nulled; not armed
	}
	/*!@~german folgende spez.Zeiten werden vorbelegt: @~english Specific times [ms] */
	/*!@~german <b>Zeiten zur 3~Netzseite :</b> @~english 3~ gridside */
	pOWN->timer[kMpmTimer_A_UNrequest].time     = 5000;                             /*! - \ref AK_UA_t_request "Zeitparameter" zur Anforderung UAN */
	pOWN->timer[kMpmTimer_A_synchronize].time   = pOWN->iAK.akA.UA_t_sync * 1000;   /*! - \ref AK_UA_t_sync "Zeitparameter" fr Synchronisation auf UAN */
	pOWN->timer[kMpmTimer_A_resynchronize].time = pOWN->iAK.akA.UA_t_resync * 1000; /*! - \ref UA_t_resync "Zeitparameter" fr Resynchronisation auf UAN */

	pOWN->timer[kMpmTimer_A_Umin1].time = pOWN->iAK.akA.UA_t_3AC_min1 * 1000; /*! - \ref AK_UA_t_3AC_min1 "Zeitparameter" fr die erste Schwelle der minimalen einphasigen Spannungsgrenze	[Second]*/
	pOWN->timer[kMpmTimer_A_Umin2].time = pOWN->iAK.akA.UA_t_3AC_min2 * 1000; /*! - \ref AK_UA_t_3AC_min2 "Zeitparameter" fr die zweite Schwelle der minimalen einphasigen Spannungsgrenze	[Second]*/
	pOWN->timer[kMpmTimer_A_Umin3].time = pOWN->iAK.akA.UA_t_3AC_min3 * 1000; /*! - \ref AK_UA_t_3AC_min3 "Zeitparameter" fr die dritte Schwelle der minimalen einphasigen Spannungsgrenze	[Second]*/
	pOWN->timer[kMpmTimer_A_Umin4].time = pOWN->iAK.akA.UA_t_3AC_min4 * 1000; /*! - \ref AK_UA_t_3AC_min4 "Zeitparameter" fr die vierte Schwelle der minimalen einphasigen Spannungsgrenze	[Second]*/
	pOWN->timer[kMpmTimer_A_Umax1].time = pOWN->iAK.akA.UA_t_3AC_max1 * 1000; /*! - \ref AK_UA_t_3AC_max1 "Zeitparameter" fr die erste Schwelle der maximalen einphasigen Spannungsgrenze	[Second]*/
	pOWN->timer[kMpmTimer_A_Umax2].time = pOWN->iAK.akA.UA_t_3AC_max2 * 1000; /*! - \ref AK_UA_t_3AC_max2 "Zeitparameter" fr die zweite Schwelle der maximalen einphasigen Spannungsgrenze	[Second]*/
	pOWN->timer[kMpmTimer_A_Umax3].time = pOWN->iAK.akA.UA_t_3AC_max3 * 1000; /*! - \ref AK_UA_t_3AC_max3 "Zeitparameter" fr die dritte Schwelle der maximalen einphasigen Spannungsgrenze	[Second]*/
	pOWN->timer[kMpmTimer_A_Fuse].time  = pOWN->iAK.akA.UA_t_3AC_fuse * 1000; /*! - \ref AK_UA_t_3AC_fuse "Zeitparameter" fr die Schwelle der Sicherungsberwachung	[Second]*/

	pOWN->timer[kMpmTimer_A_Fmax1].time = pOWN->iAK.akA.FA_t_3AC_max1 * 1000; /*! - \ref AK_FA_t_3AC_max1 "Zeitparameter" fr die erste Schwelle der maximalen Frequenz [Second]*/
	pOWN->timer[kMpmTimer_A_Fmax2].time = pOWN->iAK.akA.FA_t_3AC_max2 * 1000; /*! - \ref AK_FA_t_3AC_max2 "Zeitparameter" fr die zweite Schwelle der maximalen Frequenz [Second]*/
	pOWN->timer[kMpmTimer_A_Fmin1].time = pOWN->iAK.akA.FA_t_3AC_min1 * 1000; /*! - \ref AK_FA_t_3AC_min1 "Zeitparameter" fr die erste Schwelle der maximalen Frequenz [Second]*/
	pOWN->timer[kMpmTimer_A_Fmin2].time = pOWN->iAK.akA.FA_t_3AC_min2 * 1000; /*! - \ref AK_FA_t_3AC_min2 "Zeitparameter" fr die zweite Schwelle der maximalen Frequenz [Second]*/

	/*!@~german <b>Zeiten zur 1~Bahnseite :</b> @~english 1~ railside */
	pOWN->timer[kMpmTimer_B_synchronize].time = pOWN->iAK.akB.UB_t_sync * 1000;
	pOWN->timer[kMpmTimer_UB_releaseSupervision].time = 5000;                 // Freigabe B-Spannungsberwachung erfolgt, wenn CCS keine Notfallbehandlung durchfhrt
	pOWN->timer[kMpmTimer_B_Umin1].time = pOWN->iAK.akB.UB_t_1AC_min1 * 1000; /*! - \ref AK_UB_t_1AC_min1 "Zeitparameter" fr die erste Schwelle der minimalen einphasigen Spannungsgrenze	[Second]*/
	
	pOWN->timer[kMpmTimer_B_Umin2].time = (pOWN->iAK.akB.UB_t_1AC_min2 + pOWN->iAK.akB.UB_td_1AC_min2) * 1000; /*! - \ref AK_UB_t_1AC_min2 "Zeitparameter" fr die zweite Schwelle der minimalen einphasigen Spannungsgrenze [Second]*/
	pOWN->timer[kMpmTimer_B_Umax1].time = pOWN->iAK.akB.UB_t_1AC_max1 * 1000; /*! - \ref AK_UB_t_1AC_max1 "Zeitparameter" fr die erste Schwelle der maximalen einphasigen Spannungsgrenze	[Second]*/
	pOWN->timer[kMpmTimer_B_Umax2].time = pOWN->iAK.akB.UB_t_1AC_max2 * 1000; /*! - \ref AK_UB_t_1AC_max2 "Zeitparameter" fr die zweite Schwelle der maximalen einphasigen Spannungsgrenze	[Second]*/
	pOWN->timer[kMpmTimer_B_fault].time = 2000;                               /*! - Zeitparameter fr Verzgerung ungltiges Netz*/

	pOWN->timer[kMpmTimer_B_Fmax1].time = pOWN->iAK.akB.FB_t_1AC_max1 * 1000; /*! - \ref AK_FB_t_1AC_max1 "Zeitparameter" fr die erste Schwelle der maximalen Frequenz [Second]*/
	pOWN->timer[kMpmTimer_B_Fmax2].time = pOWN->iAK.akB.FB_t_1AC_max2 * 1000; /*! - \ref AK_FB_t_1AC_max2 "Zeitparameter" fr die zweite Schwelle der maximalen Frequenz [Second]*/
	pOWN->timer[kMpmTimer_B_Fmin1].time = pOWN->iAK.akB.FB_t_1AC_min1 * 1000; /*! - \ref AK_FB_t_1AC_min1 "Zeitparameter" fr die erste Schwelle der maximalen Frequenz [Second]*/
	pOWN->timer[kMpmTimer_B_Fmin2].time = pOWN->iAK.akB.FB_t_1AC_min2 * 1000; /*! - \ref AK_FB_t_1AC_min2 "Zeitparameter" fr die zweite Schwelle der maximalen Frequenz [Second]*/
	pOWN->timer[kMpmTimer_B_F_releaseSupervision].time = 200;

	pOWN->timer[kMpmTimer_B_THDmax].time = pOWN->iAK.akB.t_THDI_MAX * 1000; /*! - \ref AK_t_THDI_MAX "Zeitparameter" fr die maximalen THDi [Second]	*/
	pOWN->Tsmooth_B9 = pOWN->iAK.akB.IB_t_1AC_max2;                         /*! - mit \ref AK_IB_t_1AC_max2 "Glttungszeit" fr xTHDi */

	pOWN->timer[kMpmTimer_A_Usync_VLT].time = 1500; //!< Umschaltung auf Synchronisierung mit Spannung vom Vorladetrafo
}

/*! \brief KMPM_initFaultCategories
 * \anchor KMPM_initFaultCategories \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 * <b>Initialisierung der Fehlerklassen</b>
 */
static void KMPM_initFaultCategories(ZMPM_IO* pOWN) {
	YMPM_gblFaults* faultMask[14];
	faultMask[1]  = (YMPM_gblFaults*) &pOWN->cFaults.mr1;
	faultMask[2]  = (YMPM_gblFaults*) &pOWN->cFaults.mr2;
	faultMask[3]  = (YMPM_gblFaults*) &pOWN->cFaults.mr3;
	faultMask[6]  = (YMPM_gblFaults*) &pOWN->cFaults.mr6;
	faultMask[7]  = (YMPM_gblFaults*) &pOWN->cFaults.mr7;
	faultMask[8]  = (YMPM_gblFaults*) &pOWN->cFaults.mr8;
	faultMask[11] = (YMPM_gblFaults*) &pOWN->cFaults.mr11;
	faultMask[12] = (YMPM_gblFaults*) &pOWN->cFaults.mr12;
	faultMask[13] = (YMPM_gblFaults*) &pOWN->cFaults.mr13;
	//! folgende Fehler sind der R3-Fehlerkategorie zugeordnet:
	faultMask[3]->uUsync_VLT = 1; //! - \ref YMPM_gblFaults::uUsync_VLT "Syn.-Spannung von Vorladetrafo nicht bereit"

	KMPM_initFaultCategories_UAN(pOWN);
	KMPM_initFaultCategories_UAF(pOWN);
	KMPM_initFaultCategories_IA(pOWN);
	KMPM_initFaultCategories_FA(pOWN);

	KMPM_initFaultCategories_UBN(pOWN);
	KMPM_initFaultCategories_UBF(pOWN);
	KMPM_initFaultCategories_IB(pOWN);
	KMPM_initFaultCategories_FB(pOWN);

}

/* checkUA */
/*! \brief checkUA
 * \anchor KMPM_checkUA \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 * \param pSEQ2 Vorgaben der Steuerung
 *
 * berprfung der 3~Spannungen 
 */
static void KMPM_checkUA(ZMPM_IO* pOWN, ZSEQ2_IO* pSEQ2) {
	float xUAN, xUAF;
	YSFCTimer *pTimer;
	bool fTimeout;

	// Parameter
	float UA_U_3AC_min1; // Erste Schwelle der minimalen dreiphasigen Spannungsgrenze   [%]
	float UA_U_3AC_min2; // Zweite Schwelle der minimalen dreiphasigen Spannungsgrenze  [%]
	float UA_U_3AC_min3; // Drittte Schwelle der minimalen dreiphasigen Spannungsgrenze [%]
	float UA_U_3AC_max1; // Erste Schwelle der maximalendreiphasigen Spannungsgrenze    [%]
	float UA_U_3AC_max2; // Zweite Schwelle der maximalen dreiphasigen Spannungsgrenze  [%]
	float UA_U_3AC_max3; // Drittte Schwelle der maximalen dreiphasigen Spannungsgrenze [%]
	float UA_K_LE_3AC;
	float UA_dU_sync;
	// mit Zusatzparametern der Timer, verarbeitet in initTimer()

	float Unenn = pOWN->iAK.akA.akUN;
	if(Unenn > 0.0)
	{
		UA_U_3AC_min1 = (pOWN->iAK.akA.UA_U_3AC_min1/100)*Unenn;
		UA_U_3AC_min2 = (pOWN->iAK.akA.UA_U_3AC_min2/100)*Unenn;
		UA_U_3AC_min3 = (pOWN->iAK.akA.UA_U_3AC_min3/100)*Unenn;
		UA_U_3AC_max1 = (pOWN->iAK.akA.UA_U_3AC_max1/100)*Unenn;
		UA_U_3AC_max2 = (pOWN->iAK.akA.UA_U_3AC_max2/100)*Unenn;
		UA_U_3AC_max3 = (pOWN->iAK.akA.UA_U_3AC_max3/100)*Unenn;
		UA_K_LE_3AC   = (pOWN->iAK.akA.UA_K_LE_3AC/100) * ((pOWN->iAK.akA.UA_MIN_PSMb/100) * Unenn * 0.57735); // Limit * (xUA in PSMb)

		UA_dU_sync  = (pOWN->iAK.akA.UA_dU_sync/100)*Unenn;
	}
	else
	{
		pOWN->statusA.eUN = kMPM_eU_none;
		pOWN->statusUAN.ib.eUstate = kMPM_eU_none;
		pOWN->statusA.eUF = kMPM_eU_none;
		pOWN->statusUAF.ib.eUstate = kMPM_eU_none;

		return;
	}
	xUAN = pOWN->XA.xUAN;
	xUAF = pOWN->XA.xUAF;

	if( (pOWN->cA_yard.ib.cRequestUmess == kUFselect)&&
		(pOWN->cA_yard.ib.pRequestUN))
	{
		pOWN->statusUAN.ib.eUNtested = 0;
	}

	// berprfung der Netzspannungen UAN + UAF
	// Nach UAN-Anforderung kommt Zuteilung innerhalb der Zeit 'kMpmTimer_A_UNrequest'
	// ansonsten wird Spannung als schwarz erachtet
	fTimeout = 0;
	pTimer = &(pOWN->timer[kMpmTimer_A_UNrequest]);
	if((pOWN->cA_yard.ib.cRequestUmess == kUNselect) &&
		(!pOWN->statusUAN.ib.eUNallocated))
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout = SFC_TimerTimeout(pTimer);
	if(fTimeout) {
		pOWN->statusUAN.ib.eUstate = kMPM_eU_Black;
		pOWN->statusUAN.ib.eUNtested = 1;
	}

	// allokierte UAN berprfen
	if(pOWN->statusUAN.ib.eUNallocated)
	{
		if( ( xUAN > UA_U_3AC_min2 ) && ( xUAN < UA_U_3AC_max2 ) && (pOWN->mMPMA.msw.w2.b.eUAN ) )
		{
			// Spannung im gltigen Bereich
			pOWN->statusUAN.ib.eUstate = kMPM_eU_Valid;
		}
		else
		{
			// Spannung im nicht-gltigen Bereich
			pOWN->faults.ib.UAN_outOfRange = 1;

			// Haben wir ein schwarzes Netz?
			pOWN->statusUAN.ib.eUstate = (xUAN < UA_U_3AC_min3) ? kMPM_eU_Black : kMPM_eU_Fault;
		}
		pOWN->statusUAN.ib.eUNtested = 1;
	}else{pOWN->statusUAN.ib.eUNtested = 0;}
	// UAF berprfen
	if( ( xUAF > UA_U_3AC_min2 ) && ( xUAF < UA_U_3AC_max2 ) && ( pOWN->mMPMA.msw.w1.b.eUAF) )
	{
		// Spannung im gltigen Bereich
		pOWN->statusUAF.ib.eUstate = kMPM_eU_Valid;
	}
	else
	{
		// Spannung im nicht-gltigen Bereich
		pOWN->faults.ib.UAN_outOfRange = 1;

		// Haben wir ein schwarzes Netz?
		if(pSEQ2->wFeeding_checked != kFeeding_LTa){
			pOWN->statusUAF.ib.eUstate = (xUAF < UA_U_3AC_min3) ? kMPM_eU_Black : kMPM_eU_Fault;
		}else{pOWN->statusUAF.ib.eUstate = kMPM_eU_Valid;}
	}

	// Strmeldung, wenn UAF ausserhalb des Arbeitsbereichs
	if(pOWN->cA_yard.ib.eCBon) // mit geschlossenem Leistungsschalter
	{
		//! ##### berprfung auf berspannung #####
		KMPM_checkUAF_Overvoltage(pOWN);
		//! ##### berprfung auf Unterspannung #####
		KMPM_checkUAF_Undervoltage(pOWN, pSEQ2);
	}
	else
	{
		pOWN->statusUAF.ib.wShutOFFmax = 0;
		pOWN->statusUAF.ib.uShutOFFmax = 0;
		pOWN->statusUAF.ib.wShutOFFmin = 0;
		pOWN->statusUAF.ib.uShutOFFmin = 0;
		pOWN->statusUAF.ib.uStrangPSM  = 0;
	}
	if((pSEQ2->wFeeding_checked == kSeqPSM_B) ||
		!pOWN->cA_yard.ib.eCBon )
	{
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_A_Umin4]));
	}

	// Synchronberwachung
	KMPM_checkSynchA(pOWN);

	//! - Erdschlussberwachung in PSMb
	pOWN->test = 0x0;
	pOWN->ytest.value[kUA_K_LE_3AC] = UA_K_LE_3AC;
	if(pOWN->XA.xUAF1 < UA_K_LE_3AC) pOWN->test |= 0x1;
	if(pOWN->XA.xUAF2 < UA_K_LE_3AC) pOWN->test |= 0x2;
	if(pOWN->XA.xUAF3 < UA_K_LE_3AC) pOWN->test |= 0x4;
	if(pSEQ2->wFeeding_checked == kFeeding_PSMb) pOWN->test |= 0x10;
	if(pOWN->cB_yard.ib.eCBon)      pOWN->test |= 0x20;
	if(pOWN->statusUAF.ib.uESpsm)   pOWN->test |= 0x100;

	if( ((pOWN->XA.xUAF1 < UA_K_LE_3AC)   ||
		 (pOWN->XA.xUAF2 < UA_K_LE_3AC)   ||
		 (pOWN->XA.xUAF3 < UA_K_LE_3AC))  &&
		(pSEQ2->wFeeding_checked == kFeeding_PSMb) &&
		(pOWN->cB_yard.ib.eCBon) )
	{
		pOWN->statusUAF.ib.uESpsm = 1;
	}else{pOWN->statusUAF.ib.uESpsm = 0;}
}

/*! \brief checkUAF_Overvoltage
 * \anchor KMPM_checkUAF_Overvoltage \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 * UAF-berwachung auf berspannung
 * Bei berspannung erfolgt zunchst eine Warnung und anschlieend der Wechsel in den Phasenschieberbetrieb bzw. Abschaltung.
 */
static void KMPM_checkUAF_Overvoltage(ZMPM_IO* pOWN) {
	float xUAF;
	YSFCTimer *pTimer;
	bool fTimeout;
	// Parameter
	float UA_U_3AC_max1 = (pOWN->iAK.akA.UA_U_3AC_max1/100)*pOWN->iAK.akA.akUN; // Erste Schwelle der maximalendreiphasigen Spannungsgrenze	[%]
	float UA_U_3AC_max2 = (pOWN->iAK.akA.UA_U_3AC_max2/100)*pOWN->iAK.akA.akUN; // Zweite Schwelle der maximalen dreiphasigen Spannungsgrenze	[%]
	float UA_U_3AC_max3 = (pOWN->iAK.akA.UA_U_3AC_max3/100)*pOWN->iAK.akA.akUN; // Drittte Schwelle der maximalen dreiphasigen Spannungsgrenze	[%]

	xUAF = pOWN->XA.xUAF;
	// ##### berprfung auf berspannung #####
	fTimeout = 0;
	// 1. Maximalgrenze -> Warnung
	pTimer = &(pOWN->timer[kMpmTimer_A_Umax1]);
	if(xUAF > UA_U_3AC_max1)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout = SFC_TimerTimeout(pTimer);
	pOWN->statusUAF.ib.wShutOFFmax = fTimeout;

	// 2.Maximalgrenze -> Abschaltung
	fTimeout = 0;
	pTimer = &(pOWN->timer[kMpmTimer_A_Umax2]);
	if(xUAF > UA_U_3AC_max2)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout = SFC_TimerTimeout(pTimer);
	// 3.Maximalgrenze -> Abschaltung
	pTimer = &(pOWN->timer[kMpmTimer_A_Umax3]);
	if(xUAF > UA_U_3AC_max3)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout |= SFC_TimerTimeout(pTimer);
	pOWN->statusUAF.ib.uShutOFFmax = fTimeout;
}

/*! \brief KMPM_checkUAF_Undervoltage
 * \anchor KMPM_checkUAF_Undervoltage \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 * UAF-berwachung auf Unterspannung \n
 * Unterhalb eines Grenzwert fr dauerhafte drehstromseitige Netzunterspannung UAmin [kV], 
 * ist ausgehend von Nennleistung bis zu einer Spannung von UA_U_3AC_min3 eine Reduktion der auf der Bahnseite abgegebenen Leistung zulssig. \n
 * Fr Spannungen kleiner als UA_U_3AC_min3	erfolgt die Sofortabschaltung und der Umrichter geht in den Phasenschieberbetrieb.
 */
static void KMPM_checkUAF_Undervoltage(ZMPM_IO* pOWN, ZSEQ2_IO* pSEQ2) {
	float xUAF;
	YSFCTimer *pTimer;
	bool fTimeout;

	// Parameter
	float UA_U_3AC_min1 = (pOWN->iAK.akA.UA_U_3AC_min1/100)*pOWN->iAK.akA.akUN; //! Erste Schwelle der minimalen dreiphasigen Spannungsgrenze	[%]
	float UA_U_3AC_min2 = (pOWN->iAK.akA.UA_U_3AC_min2/100)*pOWN->iAK.akA.akUN; //! Zweite Schwelle der minimalen dreiphasigen Spannungsgrenze	[%]
	float UA_U_3AC_min3 = (pOWN->iAK.akA.UA_U_3AC_min3/100)*pOWN->iAK.akA.akUN; //! Drittte Schwelle der minimalen dreiphasigen Spannungsgrenze	[%]
	float UA_U_3AC_min4 = (pOWN->iAK.akA.UA_U_3AC_min4/100)*pOWN->iAK.akA.akUN/sqrt(2); //! Vierte Schwelle der minimalen dreiphasigen Spannungsgrenze zur Umschaltung auf PSMb	[%]

	xUAF = pOWN->XA.xUAF;
	//! ##### berprfung auf Unterspannung #####
	/*! erster minimaler Sperrbereich als Warnung 
	 * wird ausgeblendet, wenn der Zeitwert auf 999 s steht
	 */
	if(pOWN->iAK.akA.UA_t_3AC_min1 != 999.0) {
		fTimeout = 0;
		pTimer = &(pOWN->timer[kMpmTimer_A_Umin1]);
		if((xUAF < UA_U_3AC_min1))
		{
			if(!pTimer->armed)
			{
				SFC_TimerLoad(pTimer);
			}
		}
		else
		{
			SFC_TimerReset(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
		pOWN->statusUAF.ib.wShutOFFmin = fTimeout;
	}
	//! zweiter minimaler Sperrbereich mit variabler Zeitdauer
	KMPM_calcVarTimeForUndervoltage(pOWN);

	fTimeout = 0;
	pTimer = &(pOWN->timer[kMpmTimer_A_Umin2]);
	if((xUAF < UA_U_3AC_min2))
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout = SFC_TimerTimeout(pTimer);

	//! dritter minimaler Sperrbereich
	pTimer = &(pOWN->timer[kMpmTimer_A_Umin3]);
	if((xUAF < UA_U_3AC_min3))
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}else{
		SFC_TimerReset(pTimer);
	}
	fTimeout |= SFC_TimerTimeout(pTimer);
	pOWN->statusUAF.ib.uShutOFFmin = fTimeout;

	//! berwachung der Strangspannungen auf Unterspannung mit Umschaltung auf PSMb
	fTimeout = 0;
	pTimer = &(pOWN->timer[kMpmTimer_A_Umin4]);
	if(((pOWN->XA.xUAF1 < UA_U_3AC_min4)   ||
		(pOWN->XA.xUAF2 < UA_U_3AC_min4)   ||
		(pOWN->XA.xUAF3 < UA_U_3AC_min4))  &&
		!pOWN->Ein.cA_yard.ib.cUsync_VLT &&
		(pSEQ2->wFeeding_checked != kSeqLT_A))
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}else{
		SFC_TimerReset(pTimer);
	}
	fTimeout = SFC_TimerTimeout(pTimer);
	pOWN->statusUAF.ib.uStrangPSM = fTimeout;
}

/* calcVarTimeForUndervoltage */
/*! \brief calcVarTimeForUndervoltage
 * \anchor KMPM_calcVarTimeForUndervoltage \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 * variable Ausschaltzeit berechnen (abhngig von der Istspannung) 
 */
static void KMPM_calcVarTimeForUndervoltage(ZMPM_IO* pOWN) {
/*! variable Zeit ist abhngig von der aktuellen Unterspannung
	 mit Tvar = mx + c
			yo - yu
	 m = --------
			xo - xu
*/
float varTime;
float t0;
float m	= 0.0f;
float maxU = (pOWN->iAK.akA.UA_U_3AC_min2/100)*pOWN->iAK.akA.akUN; //! Zweite Schwelle der minimalen dreiphasigen Spannungsgrenze	[%]
float minU = (pOWN->iAK.akA.UA_U_3AC_min3/100)*pOWN->iAK.akA.akUN; //! Drittte Schwelle der minimalen dreiphasigen Spannungsgrenze	[%]
float maxT = pOWN->timer[kMpmTimer_A_Umin2].time;
float minT = pOWN->timer[kMpmTimer_A_Umin3].time;
float actU = pOWN->XA.xUAF;

if((minU<actU)&&(actU<maxU))
{
	m = (maxT-minT)/(maxU-minU);
	t0 = minU*(-1) * m + minT;

	varTime = m*actU + t0;
	if(varTime > maxT) {varTime = maxT;}
	if(varTime < minT) {varTime = minT;}
	//SFC_TimerReLoad(&pOWN->timer[kMpmTimer_A_Umin2], varTime);
}
}

/*! \brief checkUAF_1PhaseEarth
 * \return void
 * \param pOWN eigener Strukturzeiger
 * \brief UAF-berwachung auf einphasiger Erdschluss
 *
 * <b>Schutzfunktion einphasiger Erdschluss</b> \n
 *	In der Synchronisierphase wird die A-Seite auf einen einphasigen Erschluss berwacht. \n
 *	Wenn
 * - eine Strangspannung UAFx < U_3AC_min3 / 1,2247 ist und
 * - die beiden anderen Strangspannungen oberhalb des zulssigen Spannungsbereiches liegen (> U_3AC_max3 / 1,2247), \n
 * erfolgt eine Abschaltung gem R3. \n
 * Diese berwachung ist nach dem Zuschalten des A-Schalters bis zur nchsten Synchronisation zu deaktivieren.
 */
static void KMPM_checkUAF_1PhaseEarth(ZMPM_IO* pOWN) {
	float xUAF[3];
	int i;
	int earthed[3];
	int max[3];
	int result[3];

	float UA_U_3AC_min3; // Dritte Schwelle der minimalen einphasigen Spannungsgrenze	[%]
	float UA_U_3AC_max3; // Dritte Schwelle der maximalen einphasigen Spannungsgrenze	[%]
	float Unenn = pOWN->iAK.akA.akUN;
	if(Unenn > 0.0)
	{
		UA_U_3AC_min3 = (pOWN->iAK.akA.UA_U_3AC_min3/(100*1.2247))*Unenn;
		UA_U_3AC_max3 = (pOWN->iAK.akA.UA_U_3AC_max3/(100*1.2247))*Unenn;
	}
	else
	{
		pOWN->statusB.eUN = kMPM_eU_none;
		pOWN->statusB.eUF = kMPM_eU_none;
		return;
	}

	xUAF[0] = pOWN->XA.xUAF1;
	xUAF[1] = pOWN->XA.xUAF2;
	xUAF[2] = pOWN->XA.xUAF3;
	//! Mit Start der 3~ Synchronisierung
	if(pOWN->cA_yard.ib.cSynchronize)
	{
		//! erfolgt fr jede Spannung UAFx die Prfung auf Unter- bzw. berspannung. \n
		for(i=0;i<3;i++)
		{
			earthed[i] = 0;
			max[i] = 0;
			if(xUAF[i] < UA_U_3AC_min3)
			{
				earthed[i] = 10;
			}
			if(xUAF[i] > UA_U_3AC_max3)
			{
				max[i] = 1;
			}
		}
		//! Der Vergleich der momentanen Spannungsstati ergibt obiges Vergleichsergebnis
		result[0] = earthed[0] + max[1] + max[2];
		result[1] = earthed[1] + max[0] + max[2];
		result[2] = earthed[2] + max[0] + max[1];
		pOWN->statusUAF.ib.eU_1PhaseEarth = 0;
	//	if(pOWN->cA_yard.ib.eCBon)
	//	{
			for(i=0;i<3;i++)
			{
				if(result[i] == 12)
				{
					//! mit Eintrag ins Meldesystem.
					pOWN->statusUAF.ib.eU_1PhaseEarth = 1;
				}
			}
	//	}
	}
	else
	{
		pOWN->statusUAF.ib.eU_1PhaseEarth = 0;
	}
}


/*! \brief checkUAF_Fuse
 * \anchor KMPM_checkUAF_Fuse \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 * berwachung auf Sicherungsfall im Strang 
 */
static void KMPM_checkUAF_Fuse(ZMPM_IO* pOWN, ZSEQ2_IO* pSEQ2) {
	/* Schutzfunktion 14.1.3
	 * Zu Projekten, die nicht berwachte Sicherungen fr die UAF-Wandler verwenden.
	 * ------------------------------------------------------------------------------
	 * Der Sicherungsfall ist ber die Analyse der Strangspannungen zu erkennen:
	 * Wenn eine Strangspannung UAFx < U_3AC_min3 / 1,2247  ist
	 * und die beiden anderen Strangspannungen innerhalb des zulssigen Spannungsbereiches fr UAF
	 * (U_3AC_min1 / 1,2247, U_3AC_max1 / 1,2247) liegen, erfolgt eine Abschaltung gem R2.
	*/
	float xUAF[3];
	int i;
	int fuse[3];
	int normal[3];
	int result[3];
	YSFCTimer *pTimer;
	bool fTimeout;
	bool eU_fuse = 0;

	float UA_U_3AC_min1; // Dritte Schwelle der minimalen Spannungsgrenze	[%]
	float UA_U_3AC_max1; // Dritte Schwelle der maximalen Spannungsgrenze	[%]
	float Unenn = pOWN->iAK.akA.akUN;
	if(Unenn > 0.0)
	{
		UA_U_3AC_min1 = (pOWN->iAK.akA.UA_U_3AC_min3/(100*1.2247))*Unenn;
		UA_U_3AC_max1 = (pOWN->iAK.akA.UA_U_3AC_max1/(100*1.2247))*Unenn;
	}
	else
	{
		pOWN->statusUAF.ib.eUstate |= kMPM_eU_none;
		return;
	}

	xUAF[0] = pOWN->XA.xUAF1;
	xUAF[1] = pOWN->XA.xUAF2;
	xUAF[2] = pOWN->XA.xUAF3;

	// Prfung auf Unter- bzw. berspannung
	for(i=0;i<3;i++)
	{
		fuse[i]	= 0;
		normal[i] = 0;
		if(xUAF[i] < UA_U_3AC_min1)
		{
			fuse[i] = 10;
		}
		if((xUAF[i] > UA_U_3AC_min1) && (xUAF[i] < UA_U_3AC_max1))
		{
			normal[i] = 1;
		}
	}
	result[0] = fuse[0] + normal[1] + normal[2];
	result[1] = fuse[1] + normal[0] + normal[2];
	result[2] = fuse[2] + normal[0] + normal[1];

	if((pOWN->cA_yard.ib.eCBon))
	{
		for(i=0;i<3;i++)
		{
			if(result[i] == 12)
			{
				eU_fuse = 1;
			}
		}
	}
	fTimeout = 0;
	pTimer = &(pOWN->timer[kMpmTimer_A_Fuse]);
	if(eU_fuse)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout = SFC_TimerTimeout(pTimer);
	pOWN->statusUAF.ib.eU_fuse = fTimeout;
}

/*! \brief checkFA
 * \anchor KMPM_checkFA \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 * \brief berpffung der 3~Frequenz
 *
 * Es sind jeweils fr zu hohe und fr zu niedrige Frequenzen zwei projektspezifische Grenzwerte vorgesehen, \n
 * fr die eine Warnung R7 bzw. Wechsel in den Phasenschieberbetrieb R4 erfolgt. \n
 * - f0 < f_min1 oder f0 > f_max1 nach Verzgerung Tx erfolgt R7
 * - f0 < f_min2 oder f0 > f_max2 nach Verzgerung Tx erfolgt R4 
 * 
 * Diese berwachung erfolgt nur, wenn UAF angewhlt und der Netzschalter geschlossen ist.
 */
static void KMPM_checkFA(ZMPM_IO* pOWN) {
	YSFCTimer *pTimer;
	bool fTimeout;
	/*	 Netzfrequenz	*/
	float xFA;
	// Parameter
	float f_3AC_max1 = pOWN->iAK.akA.FA_f_3AC_max1; // Erste Schwelle der maximalen Frequenz [Hz]
	float f_3AC_max2 = pOWN->iAK.akA.FA_f_3AC_max2; // Zweite Schwelle der maximalen Frequenz [Hz]
	float f_3AC_min1 = pOWN->iAK.akA.FA_f_3AC_min1; // Erste Schwelle der maximalen Frequenz [Hz]
	float f_3AC_min2 = pOWN->iAK.akA.FA_f_3AC_min2; // Zweite Schwelle der maximalen Frequenz [Hz]

	xFA = pOWN->XA.xfA;
	pOWN->statusFA.ib.eState = kMPM_f_valid;

	// ### berprfung auf Minimum ###
	fTimeout = 0;
	// 1. Minimalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_A_Fmin1]);
	if(xFA < f_3AC_min1)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
		if(fTimeout)
		{
			pOWN->statusFA.ib.eState = kMPM_wf_min;
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}

	fTimeout = 0;
	// 2. Minimalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_A_Fmin2]);
	if(xFA < f_3AC_min2)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
		if(fTimeout)
		{
			pOWN->statusFA.ib.eState = kMPM_uf_min;
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}

	// ### berprfung auf Maximum ###
	fTimeout = 0;
	// 1. Maximalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_A_Fmax1]);
	if((xFA > f_3AC_max1))
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
		if(fTimeout)
		{
			pOWN->statusFA.ib.eState = kMPM_wf_max;
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout = 0;
	// 2. Maximalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_A_Fmax2]);

	if((xFA > f_3AC_max2))
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
		if(fTimeout)
		{
			pOWN->statusFA.ib.eState = kMPM_uf_max;
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	// Frequenzfehlermeldungen ignorieren, wenn undefinierter Zustand
	if(!((pOWN->cA_yard.ib.cRequestUmess == kUNselect) || pOWN->cA_yard.ib.eCBon))
	{
		pOWN->statusFA.ib.eState = kMPM_f_none;
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_A_Fmin1]));
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_A_Fmin2]));
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_A_Fmax1]));
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_A_Fmax2]));
	}

	// zur Anzeige
	if(pOWN->statusFA.ib.eState == kMPM_uf_max || pOWN->statusFA.ib.eState == kMPM_uf_min)
	{
		pOWN->statusFA.ib.opcState = kVaColor_TRIP;
	}
	else if(pOWN->statusFA.ib.eState == kMPM_wf_max || pOWN->statusFA.ib.eState == kMPM_wf_min)
	{
		pOWN->statusFA.ib.opcState = kVaColor_WARN;
	}
	else
	{
		pOWN->statusFA.ib.opcState = kVaColor_OK;
	}
}

/*! \brief checkSynchA
 * \anchor KMPM_checkSynchA \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 * Zum Schliessen des A-Schalters werden die UAN und UAF auf Gleichheit berprft.
 * Hierzu muss die Differenz zwischen Innen- und Aussenspannung kleiner als die parametrierte Grenze \ref AK_UA_dU_3AC_sync sein und \n
 */
static void KMPM_checkSynchA(ZMPM_IO* pOWN) {
	YSFCTimer *pTimer;
	bool fTimeout;
	float dU;
	float xUAN		= pOWN->XA.xUAN1;
	float xUAF		= pOWN->XA.xUAF;
	// Parameter
	float Unenn		= pOWN->iAK.akA.akUN;
	float dU_sync	= (pOWN->iAK.akA.UA_dU_sync/100)*Unenn;

	// Synchronberwachung
	dU = xUAN - xUAF;
	dU = fabs(dU);
	if((dU < dU_sync) && (pOWN->statusUAF.ib.eUstate == kMPM_eU_Valid)) pOWN->statusUAF.ib.eSynchron = 1;

	fTimeout = 0;
	// Synchronisation nach dT abbrechen
	pTimer = &(pOWN->timer[kMpmTimer_A_synchronize]);
	if((pOWN->cA_yard.ib.cSynchronize)&& (dU > dU_sync) )//&&((dU > dU_sync)||(xFA < f_min2)) )
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	pOWN->statusUAF.ib.uNotSynchron = fTimeout;
	if(fTimeout) pOWN->statusUAF.ib.eSynchron = 0;

}

/*! \brief KMPM_checkUB
 * \anchor KMPM_checkUB \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 * \param pSEQ2 Vorgaben der Steuerung
 *
 * \brief berwachung der 1~Spannungen UBN und UBF
 */
static void KMPM_checkUB(ZMPM_IO* pOWN, ZSEQ2_IO* pSEQ2, ZCCS_IO* pCCS) {
	float	 xUBN, xUBF;
	YSFCTimer	*pTimer;
	bool		 fTimeout;
	//! Parameter
	float UB_U_1AC_min1; //!< - Erste Schwelle der minimalen einphasigen Spannungsgrenze	[%]
	float UB_U_1AC_min2; //!< - Zweite Schwelle der minimalen einphasigen Spannungsgrenze	[%]
	float UB_U_1AC_min3; //!< - Drittte Schwelle der minimalen einphasigen Spannungsgrenze	[%]
	float UB_U_1AC_max1; //!< - Erste Schwelle der maximaleneinphasigen Spannungsgrenze	[%]
	float UB_U_1AC_max2; //!< - Zweite Schwelle der maximalen einphasigen Spannungsgrenze	[%]
	float UB_K_LE_1AC;
	float Unenn = pOWN->iAK.akB.akUN;
	if(Unenn > 0.0)
	{
		UB_U_1AC_min1 = (pOWN->iAK.akB.UB_U_1AC_min1/100)*Unenn;
		UB_U_1AC_min2 = (pOWN->iAK.akB.UB_U_1AC_min2/100)*Unenn;
		UB_U_1AC_min3 = (pOWN->iAK.akB.UB_U_1AC_min3/100)*Unenn;
		UB_U_1AC_max1 = (pOWN->iAK.akB.UB_U_1AC_max1/100)*Unenn;
		UB_U_1AC_max2 = (pOWN->iAK.akB.UB_U_1AC_max2/100)*Unenn;
		UB_K_LE_1AC   = (pOWN->iAK.akB.UB_K_LE_1AC/100) * ((pOWN->iAK.akB.UB_MIN_PSMa/100) * Unenn);
	}
	else
	{
		pOWN->statusB.eUN = kMPM_eU_none;
		pOWN->statusUBN.ib.eUstate = kMPM_eU_none;
		pOWN->statusB.eUF = kMPM_eU_none;
		pOWN->statusUBF.ib.eUstate = kMPM_eU_none;
		return;
	}
	xUBN = pOWN->XB.xUBN;
	xUBF = pOWN->XB.xUBF;

	// berprfung der Bahnspannungen UBN + UBF
	// ##### allokierte UBN berprfen #####
	if(pOWN->statusUBN.ib.eUNallocated)
	{
		if( ( xUBN > UB_U_1AC_min2 ) && ( xUBN < UB_U_1AC_max2 ) )
		{
			// Spannung im gltigen Bereich
			pOWN->statusUBN.ib.eUstate = kMPM_eU_Valid;
		}
		else
		{
			// Spannung im nicht-gltigen Bereich
			// Haben wir ein schwarzes Netz?
			pOWN->statusUBN.ib.eUstate = ( xUBN < UB_U_1AC_min3 ) ? kMPM_eU_Black : kMPM_eU_Fault;
		}
		pOWN->statusUBN.ib.eUNtested = 1;
		// Synchronberwachung
		KMPM_checkSynchB(pOWN);
	}else {
		pOWN->statusUBF.ib.uNotSynchron = 0;
		pOWN->statusUBN.ib.eUNtested = 0;
	}

	// ##### UBF berprfen #####
	fTimeout = 0;
	pTimer = &(pOWN->timer[kMpmTimer_B_fault]);
	if( ( xUBF > UB_U_1AC_min2 ) && ( xUBF < UB_U_1AC_max2 ) )
	{
		// Spannung im gltigen Bereich
		pOWN->statusUBF.ib.eUstate = kMPM_eU_Valid;
		SFC_TimerReset(pTimer);
	}
	else
	{
		// Spannung im nicht-gltigen Bereich
		// Haben wir ein schwarzes Netz?
		if((pSEQ2->wFeeding_checked != kFeeding_LTb)&&
			pOWN->cB_yard.ib.cSuperviseUBmin)
		{

			if(xUBF < UB_U_1AC_min3){
				pOWN->statusUBF.ib.eUstate = kMPM_eU_Black;
				SFC_TimerReset(pTimer);
				fTimeout = 0;
			}else{
				if(pOWN->cB_yard.ib.eCBon && !pTimer->armed)
				{
					SFC_TimerLoad(pTimer);
				}
				fTimeout = SFC_TimerTimeout(pTimer);

				if(fTimeout)  pOWN->statusUBF.ib.eUstate = kMPM_eU_Fault;
			}
		}else{pOWN->statusUBF.ib.eUstate = kMPM_eU_Valid;}
	}

	// Strmeldung, wenn UBF ausserhalb des Arbeitsbereichs
	if(pOWN->cB_yard.ib.eCBon) // mit geschlossenem Leistungsschalter
	{
		//! - ##### berprfung auf berspannung #####
		KMPM_checkUBF_Overvoltage(pOWN);
		//! - ##### berprfung auf Unterspannung #####
		KMPM_checkUBF_Undervoltage(pOWN);
	}
	else
	{
		//UBF-berwachung ausblenden
		pOWN->statusUBF.ib.wShutOFFmax = 0;
		pOWN->statusUBF.ib.uShutOFFmax = 0;
		pOWN->statusUBF.ib.wShutOFFmin = 0;
		pOWN->statusUBF.ib.uShutOFFmin = 0;
		// UBF-Zeitstufen zurcksetzen
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_B_Umax1]));
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_B_Umax2]));
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_B_Umin1]));
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_B_Umin2]));
	}
	// Mindestspannung im PSMa / berwachung auf Erdschluss
	if( (pSEQ2->wFeeding_checked == kFeeding_PSMa) &&
		(pOWN->cA_yard.ib.eCBon )	&&
	   ((pOWN->XB.xUBF_1r < UB_K_LE_1AC/2) || (pOWN->XB.xUBF_2r < UB_K_LE_1AC/2)) )
	{
		pOWN->statusUBF.ib.uESpsm = 1;
	}else{pOWN->statusUBF.ib.uESpsm = 0;}
	pOWN->ytest.value[kUB_K_LE_1AC] = UB_K_LE_1AC;
}

/*! \brief checkUBF_Overvoltage
 * \anchor KMPM_checkUBF_Overvoltage \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 *	berprfung des UBF auf berspannung
 */
static void KMPM_checkUBF_Overvoltage(ZMPM_IO* pOWN) {
	float xUBF;
	YSFCTimer *pTimer;
	bool fTimeout;
	// Parameter
	float UB_U_1AC_max1; // Erste Schwelle der maximaleneinphasigen Spannungsgrenze	[%]
	float UB_U_1AC_max2; // Zweite Schwelle der maximalen einphasigen Spannungsgrenze	[%]

	float Unenn = pOWN->iAK.akB.akUN;
	if(Unenn > 0.0)
	{
		UB_U_1AC_max1 = (pOWN->iAK.akB.UB_U_1AC_max1/100)*Unenn;
		UB_U_1AC_max2 = (pOWN->iAK.akB.UB_U_1AC_max2/100)*Unenn;
	}
	else
	{
		pOWN->statusB.eUN = kMPM_eU_none;
		pOWN->statusUBN.ib.eUstate = kMPM_eU_none;
		pOWN->statusB.eUF = kMPM_eU_none;
		pOWN->statusUBF.ib.eUstate = kMPM_eU_none;
		return;
	}
	xUBF = pOWN->XB.xUBF;

	// berprfung der Bahnspannungen UBF
	// Strmeldung, wenn UBF oberhalb des Arbeitsbereichs
	fTimeout = 0;
	// 1. Maximalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_B_Umax1]);
	if(xUBF > UB_U_1AC_max1)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout = SFC_TimerTimeout(pTimer);
	pOWN->statusUBF.ib.wShutOFFmax = fTimeout;
	// 2.Maximalgrenze
	fTimeout = 0;
	pTimer = &(pOWN->timer[kMpmTimer_B_Umax2]);
	if(xUBF > UB_U_1AC_max2)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout |= SFC_TimerTimeout(pTimer);
	pOWN->statusUBF.ib.uShutOFFmax = fTimeout;
}

/* checkUBF_Undervoltage */
/*! \brief checkUBF_Undervoltage
 * \anchor KMPM_checkUBF_Undervoltage \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 *	berprfung des UBF auf Unterspannung
 */
static void KMPM_checkUBF_Undervoltage(ZMPM_IO* pOWN) {
	float xUBF;
	YSFCTimer *pTimer;
	bool fTimeout;
	// Parameter
	float UB_U_1AC_min1; // Erste Schwelle der minimalen einphasigen Spannungsgrenze	[%]
	float UB_U_1AC_min2; // Zweite Schwelle der minimalen einphasigen Spannungsgrenze	[%]
	float UB_U_1AC_min3; // Drittte Schwelle der minimalen einphasigen Spannungsgrenze	[%]

	float Unenn = pOWN->iAK.akB.akUN;
	if(Unenn > 0.0)
	{
		UB_U_1AC_min1 = (pOWN->iAK.akB.UB_U_1AC_min1/100)*Unenn;
		UB_U_1AC_min2 = (pOWN->iAK.akB.UB_U_1AC_min2/100)*Unenn;
		UB_U_1AC_min3 = (pOWN->iAK.akB.UB_U_1AC_min3/100)*Unenn;
	}
	else
	{
		pOWN->statusB.eUN = kMPM_eU_none;
		pOWN->statusUBN.ib.eUstate = kMPM_eU_none;
		pOWN->statusB.eUF = kMPM_eU_none;
		pOWN->statusUBF.ib.eUstate = kMPM_eU_none;
		return;
	}
	xUBF = pOWN->XB.xUBF;

	// Strmeldung, wenn UBF unterhalb des Arbeitsbereichs
	//! - \ref AK_UB_U_1AC_min1 "1. minimaler Sperrbereich" fhrt nach \ref AK_UB_t_1AC_min1 "T1" zur R2-Abschaltung
	fTimeout = 0;
	if(pOWN->iAK.akB.UB_t_1AC_min1 != 999.0) {
		pTimer = &(pOWN->timer[kMpmTimer_B_Umin1]);
		if( (xUBF < UB_U_1AC_min3)||
			(xUBF > UB_U_1AC_min1)||
			!pOWN->cB_yard.ib.cSuperviseUBmin )
		{
			SFC_TimerReset(pTimer);
		}
		else
		{
			if(!pTimer->armed)
			{
				SFC_TimerLoad(pTimer);
			}
		}
		fTimeout = SFC_TimerTimeout(pTimer);
		pOWN->statusUBF.ib.wShutOFFmin = fTimeout;
	}
	//! - \ref AK_UB_U_1AC_min2 "2. minimaler Sperrbereich" fhrt nach \ref AK_UB_t_1AC_min2 "T2" zur R2-Abschaltung
	fTimeout = 0;
	pTimer = &(pOWN->timer[kMpmTimer_B_Umin2]);
	//if((xUBF < UB_U_1AC_min3)||(xUBF > UB_U_1AC_min2))
	if( (xUBF > UB_U_1AC_min2)||
		!pOWN->cB_yard.ib.cSuperviseUBmin )
	{
		SFC_TimerReset(pTimer);
	}
	else
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
	}
	fTimeout |= SFC_TimerTimeout(pTimer);
	pOWN->statusUBF.ib.uShutOFFmin = fTimeout;
}

/*! \brief checkIB
 * \anchor KMPM_checkIB \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 * \param pCCS Zeiger auf Stromregelung KCCS
 *
 * Ist der Oberschwingungsanteil im Bahnstrom zu hoch wird eine Meldung generiert. \n
 * Die Oberschwingungsstrme werden auf den Grundschwingungsnennstrom bezogen.\n
 * Diese zeitlich verzgerte berwachung wird nur durchgefhrt, wenn die CCS an Strombegrenzung arbeitet.
 */
static void KMPM_checkIB(ZMPM_IO* pOWN,ZCCS_IO* pCCS) {
	float xTHDi;
	YSFCTimer *pTimer;
	bool fTimeout;
	float rama;
	// Parameter
	float THD_IB_MAX = pOWN->iAK.akB.THDI_MAX;
	// THD-berwachung nur bei berlastung
	xTHDi = (pOWN->XB.xTHDibf);// * pOWN->XB.xIBFF / pOWN->iAK.akB.akIUE;
	// Glttung
	rama = (pOWN->Tabtast<pOWN->Tsmooth_B9) ? pOWN->Tabtast/pOWN->Tsmooth_B9 : 1; // Begrenzung der dt-nderung, wenn Zyklus lnger als Restrampe
	pOWN->xI_B9 += (rama *(pOWN->XB.xIBF - pOWN->xI_B9));

	// ##### berprfung THDi #####
	fTimeout = 0;
	// Maximalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_B_THDmax]);
	if((xTHDi > THD_IB_MAX)&&
		(pCCS->sysStatus.mCurrLimActiv))
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	pOWN->statusIB.ib.wTHDImax = fTimeout;
}

/*! \brief KMPM_checkFB berwachung der Bahnfrequenz
 * \return void
 * \param pOWN eigener Strukturzeiger
 * \param releaseUEB Freigabe Abschaltberwachung
 *
 * Es sind jeweils fr zu hohe und fr zu niedrige Frequenzen zwei projektspezifische Grenzwerte vorgesehen, \n
 * fr die eine Warnung R7 bzw. eine Abschaltung R3 erfolgt.
 * - f10 < f_1AC_min1 oder f10 > f_1AC_max1 nach Verzgerung Tx erfolgt R7
 * - f10 < f_1AC_min2 oder f10 > f_1AC_max2 nach Verzgerung Tx erfolgt R3
 * 
 * Diese berwachung erfolgt nur, wenn UBF angewhlt und der Bahnschalter geschlossen ist.
 */
static void KMPM_checkFB(ZMPM_IO* pOWN,bool releaseUEB) {
	YSFCTimer *pTimer;
	bool fTimeout;
	/*	 Bahnfrequenz	 */
	float xFB;
	// Parameter
	float f_1AC_max1 = pOWN->iAK.akB.FB_f_1AC_max1; // Erste Schwelle der maximalen Frequenz [Hz]
	float f_1AC_max2 = pOWN->iAK.akB.FB_f_1AC_max2; // Zweite Schwelle der maximalen Frequenz [Hz]
	float f_1AC_min1 = pOWN->iAK.akB.FB_f_1AC_min1; // Erste Schwelle der maximalen Frequenz [Hz]
	float f_1AC_min2 = pOWN->iAK.akB.FB_f_1AC_min2; // Zweite Schwelle der maximalen Frequenz [Hz]

	xFB = pOWN->XB.xfB;
	pOWN->statusFB.ib.eState = kMPM_f_valid;

	// mit Freigabe der Abschaltberwachungen (nur, wenn CCS keine KS-Notfallbehandlung durchfhrt)
	fTimeout = 0;
	// 1. Minimalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_B_F_releaseSupervision]);
	if(releaseUEB)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	pOWN->statusFB.ib.releaseUeb = (fTimeout) ? 1:0;

	// ### berprfung auf Minimum ###
	fTimeout = 0;
	// 1. Minimalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_B_Fmin1]);
	if((xFB < f_1AC_min1))
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
		if(fTimeout)
		{
			pOWN->statusFB.ib.eState = kMPM_wf_min;
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout = 0;
	// 2. Minimalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_B_Fmin2]);
	if((xFB < f_1AC_min2) && pOWN->statusFB.ib.releaseUeb)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
		if(fTimeout)
		{
			pOWN->statusFB.ib.eState = kMPM_uf_min;
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}

	// ### berprfung auf Maximum ###
	fTimeout = 0;
	// 1. Maximalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_B_Fmax1]);
	if((xFB > f_1AC_max1))
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
		if(fTimeout)
		{
			pOWN->statusFB.ib.eState = kMPM_wf_max;
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	fTimeout = 0;
	// 2. Maximalgrenze
	pTimer = &(pOWN->timer[kMpmTimer_B_Fmax2]);
	if((xFB > f_1AC_max2) && pOWN->statusFB.ib.releaseUeb)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
		if(fTimeout)
		{
			pOWN->statusFB.ib.eState = kMPM_uf_max;
		}
	}
	else
	{
		SFC_TimerReset(pTimer);
	}

	// Frequenzfehlermeldungen ignorieren, wenn undefinierter Zustand
	if(!((pOWN->cB_yard.ib.cRequestUmess == kUNselect) || pOWN->cB_yard.ib.eCBon) )
	{
		pOWN->statusFB.ib.eState = kMPM_f_none;
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_B_Fmin1]));
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_B_Fmin2]));
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_B_Fmax1]));
		SFC_TimerReset(&(pOWN->timer[kMpmTimer_B_Fmax2]));
	}

	// zur Anzeige
	if(pOWN->statusFB.ib.eState == kMPM_uf_max || pOWN->statusFB.ib.eState == kMPM_uf_min)
	{
		pOWN->statusFB.ib.opcState = kVaColor_TRIP;
	}
	else if(pOWN->statusFB.ib.eState == kMPM_wf_max || pOWN->statusFB.ib.eState == kMPM_wf_min)
	{
		pOWN->statusFB.ib.opcState = kVaColor_WARN;
	}
	else
	{
		pOWN->statusFB.ib.opcState = kVaColor_OK;
	}
}

/*! \brief checkSynchB
 * \anchor KMPM_checkSynchB \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 * Zum Schliessen des B-Schalters werden die UBN und UBF auf Gleichheit berprft.
 * Hierzu muss die Differenz zwischen Innen- und Aussenspannung kleiner als die parametrierte Grenze AK_UB_dU_1AC_sync sein.
 */
static void KMPM_checkSynchB(ZMPM_IO* pOWN) {
	float xUBN, xUBF;
	YSFCTimer *pTimer;
	bool fTimeout;
	float dU;
	// Parameter
	float Unenn = pOWN->iAK.akB.akUN;
	float dU_sync = (pOWN->iAK.akB.UB_dU_sync/100)*Unenn;

	xUBN = pOWN->XB.xUBN;
	xUBF = pOWN->XB.xUBF;

	// Synchronberwachung
	dU = xUBN - xUBF;
	dU = fabs(dU);

	if((dU < dU_sync) && (pOWN->statusUBF.ib.eUstate == kMPM_eU_Valid) && (pOWN->statusFB.ib.eState == kMPM_f_valid))
	{
		pOWN->statusUBF.ib.eSynchron = 1;
	}
	else{pOWN->statusUBF.ib.eSynchron = 0;}

	fTimeout = 0;
	// Synchronisation nach dT abbrechen
	pTimer = &(pOWN->timer[kMpmTimer_B_synchronize]);
	if(pOWN->cB_yard.ib.cSynchronize)
	{
		if(!pTimer->armed)
		{
			SFC_TimerLoad(pTimer);
		}
		fTimeout = SFC_TimerTimeout(pTimer);
	}
	else
	{
		SFC_TimerReset(pTimer);
	}
	pOWN->statusUBF.ib.uNotSynchron = (fTimeout) ? !pOWN->statusUBF.ib.eSynchron : 0;
}

/*! \brief calcEfficiency
 * \anchor KMPM_calcEfficiency \n
 * \return void
 * \param pOWN eigener Strukturzeiger
 *
 * Berechnung cosP
 */
static void KMPM_calcEfficiency(ZMPM_IO* pOWN) {
	float xEfficiency;
	int mode;
	float rama;
	int facA = (pOWN->XA.xPA < 0.0) ? -1 : 1;
	int facB = (pOWN->XB.xPB < 0.0) ? -1 : 1;
	mode = facA + facB;

	switch (mode){
		case (2):
			if(pOWN->XA.xPA != 0.0)
			{
				xEfficiency = (pOWN->XB.xPB / pOWN->XA.xPA);
			}
			else
			{
				xEfficiency = 1.0;
			}
		break;

		case (-2):
			if(pOWN->XB.xPB != 0.0)
			{
				xEfficiency = (pOWN->XA.xPA / pOWN->XB.xPB);
			}
			else
			{
				xEfficiency = 1.0;
			}
		break;

		case (0):
		default:
				xEfficiency = 0.0;
		break;
	}
	// Glttung
	rama = (pOWN->Tabtast<pOWN->Tsmooth_EF) ? pOWN->Tabtast/pOWN->Tsmooth_EF : 1; // Begrenzung der dt-nderung, wenn Zyklus lnger als Restrampe
	pOWN->xEfficiency += rama * (xEfficiency - pOWN->xEfficiency);
}

static void KMPM_prepareIPC1(ZMPM_IO* pOWN) {
	YMPM_IPC1	*pIPC1;
	pIPC1 = (struct YMPM_IPC1*)&pOWN->ipc1;
	pIPC1->cA_yard = pOWN->cA_yard;
	pIPC1->cB_yard = pOWN->cB_yard;
	pIPC1->statusA = pOWN->statusA;
	pIPC1->statusB = pOWN->statusB;
}

static void KMPM_readIPC1(ZMPM_IO* pOWN) {
	YMPM_IPC1	*pIPC1;
	pIPC1 = &pOWN->ipc1;
	pOWN->cA_yard = pIPC1->cA_yard;
	pOWN->cB_yard = pIPC1->cB_yard;
	pOWN->statusA = pIPC1->statusA;
	pOWN->statusB = pIPC1->statusB;
}

static void KMPM_prepareIPC2(ZMPM_IO* pOWN) {
	YMPM_IPC2	*pIPC2;
	pIPC2 = (struct YMPM_IPC2*)&pOWN->ipc2;
	memcpy(&(pOWN->ipc2.XA) ,(void*)&(pOWN->XA) ,sizeof(YMPM_XA));
	memcpy(&(pOWN->ipc2.XB) ,(void*)&(pOWN->XB) ,sizeof(YMPM_XB));
	pIPC2->mMPMA.dw			= pOWN->mMPMA.dw;
	pIPC2->mMPMB.dw			= pOWN->mMPMB.dw;
	pIPC2->eVB0		     = pOWN->eVB0;
	pIPC2->eVB1		     = pOWN->eVB1;
	pIPC2->statusUAN     = pOWN->statusUAN.iw;
	pIPC2->xFaultsUAN.iw = pOWN->faultUAN.actFaults;
	pIPC2->statusUAF     = pOWN->statusUAF.iw;
	pIPC2->xFaultsUAF.iw = pOWN->faultUAF.actFaults;
	pIPC2->statusIA	     = pOWN->statusIA.iw ;
	pIPC2->xFaultsIA.iw  = pOWN->faultIA.actFaults;
	pIPC2->statusFA	     = pOWN->statusFA.iw ;
	pIPC2->xFaultsFA.iw  = pOWN->faultFA.actFaults;
	pIPC2->statusUBN     = pOWN->statusUBN.iw;
	pIPC2->xFaultsUBN.iw = pOWN->faultUBN.actFaults;
	pIPC2->statusUBF     = pOWN->statusUBF.iw;
	pIPC2->xFaultsUBF.iw = pOWN->faultUBF.actFaults;
	pIPC2->statusIB	     = pOWN->statusIB.iw ;
	pIPC2->xFaultsIB.iw  = pOWN->faultIB.actFaults;
	pIPC2->statusFB	     = pOWN->statusFB.iw ;
	pIPC2->xFaultsFB.iw  = pOWN->faultFB.actFaults;
}

static void KMPM_readIPC2(ZMPM_IO* pOWN) {
	YMPM_IPC2	*pIPC2;
	pIPC2 = (struct YMPM_IPC2*)&pOWN->ipc2;
	memcpy(&(pOWN->XA) ,(void*)&(pIPC2->XA) ,sizeof(YMPM_XA));
	memcpy(&(pOWN->XB) ,(void*)&(pIPC2->XB) ,sizeof(YMPM_XB));
	pOWN->statusUAN.iw	= pIPC2->statusUAN;
	pOWN->xFaultsUAN.iw	= pIPC2->xFaultsUAN.iw;
	pOWN->statusUAF.iw	= pIPC2->statusUAF;
	pOWN->xFaultsUAF.iw	= pIPC2->xFaultsUAF.iw;
	pOWN->statusIA.iw	= pIPC2->statusIA ;
	pOWN->xFaultsIA.iw	= pIPC2->xFaultsIA.iw;
	pOWN->statusFA.iw	= pIPC2->statusFA;
	pOWN->xFaultsFA.iw	= pIPC2->xFaultsFA.iw;
	pOWN->statusUBN.iw	= pIPC2->statusUBN;
	pOWN->xFaultsUBN.iw	= pIPC2->xFaultsUBN.iw;
	pOWN->statusUBF.iw	= pIPC2->statusUBF;
	pOWN->xFaultsUBF.iw	= pIPC2->xFaultsUBF.iw;
	pOWN->statusIB.iw	= pIPC2->statusIB ;
	pOWN->xFaultsIB.iw	= pIPC2->xFaultsIB.iw;
	pOWN->statusFB.iw	= pIPC2->statusFB ;
	pOWN->xFaultsFB.iw	= pIPC2->xFaultsFB.iw;

	pOWN->mMPMA.dw = pOWN->ipc2.mMPMA.dw;
	pOWN->mMPMB.dw = pOWN->ipc2.mMPMB.dw;
	pOWN->eVB0 = pIPC2->eVB0;
	pOWN->eVB1 = pIPC2->eVB1;
}

/*! \brief KMPM_readPerCommands_A
 * \anchor KMPM_readPerCommands_A
 * \return void
 * \param pOWN eigener Strukturzeiger
 * \param ica  eingelesenes Kommandowort
 *	
 * Das CFC-Kommandowort wird auf das intern wirksame umrangiert
 */
static void KMPM_readPerCommands_A(ZMPM_IO* pOWN, uint16 ica) {
	U_MPM_CfcCommands cfcCommand;
	YSFCTimer *pTimer;
	bool fTimeout;
	cfcCommand.iw = ica;
	// Kommandos werden gelesen am Eingangskonnektor ICA oder beschrieben von den Peripheriebausteinen
	if(cfcCommand.ib.cPLL_Hold) {
		  pOWN->Ein.cA_yard.ib.cSelectPLL = 3;
	}else{pOWN->Ein.cA_yard.ib.cSelectPLL = 0;}

	pOWN->cA_yard.ib.cSelectOFFSET  = cfcCommand.ib.cOFFSET;
	pOWN->cA_yard.ib.cSuperviseFreq = cfcCommand.ib.cSuperviseFreq;
	pOWN->cA_yard.ib.cSynchronize   = cfcCommand.ib.cSynchronize;
	pOWN->cA_yard.ib.eCBon          = cfcCommand.ib.eCBon;
	pOWN->cA_yard.ib.pRequestUN     = cfcCommand.ib.pRequestUN;

	// Anforderung und Synchronisierung
	pOWN->cA_yard.ib.eUNallocated   = (pOWN->cA_yard.ib.pRequestUN) ? (pOWN->Ein.cA_yard.ib.eUNallocated) : 1; // Anforderung der Messstelle (wenn notwendig)
	if(cfcCommand.ib.cUsync_VLT)    pOWN->Ein.cA_yard.ib.cUsync_VLT = 1;
	if(cfcCommand.ib.cUsync_NRM)    pOWN->Ein.cA_yard.ib.cUsync_VLT = 0;
	if(cfcCommand.ib.cRequestUxN)   pOWN->cA_yard.ib.cRequestUmess = kUNselect;
	if(cfcCommand.ib.cRequestUxF)   pOWN->cA_yard.ib.cRequestUmess = kUFselect;

	// Umschaltung UAN - UAF unter Beachtung des CCS-Status
	if(cfcCommand.ib.cSelectUF)     pOWN->Ein.cA_yard.ib.cSelectUmess = kUFselect;
	if(cfcCommand.ib.cSelectUN && pOWN->cA_yard.ib.eUNallocated) pOWN->Ein.cA_yard.ib.cSelectUmess = kUNselect;

	// Umschaltung auf Synchronisierung auf Spannung vom Vorladetrafo
	fTimeout = 0;
	pTimer = &(pOWN->timer[kMpmTimer_A_Usync_VLT]);
	if(pOWN->Ein.cA_yard.ib.cUsync_VLT) {
		if(!pTimer->armed) {
			SFC_TimerLoad(pTimer);
		}
	}else{
		SFC_TimerReset(pTimer);
	}
	fTimeout = SFC_TimerTimeout(pTimer);
	if(fTimeout && !pOWN->Ein.cA_yard.ib.eUsync_VLT){
		pOWN->faults.ib.uUsync_VLT  = 1;
		pOWN->Ein.cA_yard.ib.cUsync_VLT = 0;
	}
	pOWN->cA_yard.ib.cUsync_VLT = (pOWN->Ein.cA_yard.ib.cUsync_VLT && pOWN->Ein.cA_yard.ib.eUsync_VLT) ? 1:0;
}

/*! \brief KMPM_readPerCommands_B
 * \anchor KMPM_readPerCommands_B
 * \return void
 * \param pOWN eigener Strukturzeiger
 * \param icb  eingelesenes Kommandowort
 *	
 * Das CFC-Kommandowort wird auf das intern wirksame umrangiert
 */
static void KMPM_readPerCommands_B(ZMPM_IO* pOWN, uint16 icb, ZSEQ_IO* pSEQ, ZREF_IO* pREF) {
	U_MPM_CfcCommands cfcCommand;
	cfcCommand.iw = icb;
	// Kommandos werden gelesen am Eingangskonnektor ICB oder beschrieben von den Peripheriebausteinen
	pOWN->cB_yard.ib.pRequestUN     = cfcCommand.ib.pRequestUN;
	pOWN->cB_yard.ib.cSelectOFFSET  = cfcCommand.ib.cOFFSET;
	pOWN->cB_yard.ib.cSuperviseFreq = cfcCommand.ib.cSuperviseFreq;
	pOWN->cB_yard.ib.cSynchronize	= pOWN->Ein.cB_yard.ib.cSynchronize;
	pOWN->cB_yard.ib.eCBon			= cfcCommand.ib.eCBon;
	// Anforderung und Synchronisierung
	pOWN->cB_yard.ib.eUNallocated = (pOWN->cB_yard.ib.pRequestUN) ? (pOWN->Ein.cB_yard.ib.eUNallocated) : 1; // Anforderung der Messstelle, wenn notwendig
	if(cfcCommand.ib.cRequestUxN)    pOWN->cB_yard.ib.cRequestUmess = kUNselect;
	if(cfcCommand.ib.cRequestUxF)    pOWN->cB_yard.ib.cRequestUmess = kUFselect;
	if(cfcCommand.ib.cSelectUF)      pOWN->cB_yard.ib.cSelectUmess = kUFselect;
	if(cfcCommand.ib.cSelectUN && pOWN->cB_yard.ib.eUNallocated) pOWN->cB_yard.ib.cSelectUmess = kUNselect;
	if(cfcCommand.ib.cPLL_Hold) 
	{
		pOWN->Ein.cB_yard.ib.cSelectPLL = 3;
	}else{pOWN->Ein.cB_yard.ib.cSelectPLL = 0;}

}

/*! \brief KMPM_initFaultCategories_UAN
 * \return void
 * \param pOWN eigener Strukturzeiger
 *	
 * Fehlerkategorien zur UAN
 */
static void KMPM_initFaultCategories_UAN(ZMPM_IO* pOWN) {
	UMPM_voltage* cat6;
	UMPM_voltage* cat7;
	cat6 = (UMPM_voltage*) &pOWN->faultUAN.mr6;
	cat6->ib.eU_fuse    = 1;
	cat6->ib.eU_MCB_VT = 1;

	cat7 = (UMPM_voltage*) &pOWN->faultUAN.mr7;
	cat7->ib.wShutOFFmin = 1;
	cat7->ib.wShutOFFmax = 1;
}


/*! \brief KMPM_initFaultCategories_UAF
 * \return void
 * \param pOWN eigener Strukturzeiger
 *	
 * Fehlerkategorien zur UAF
 */
static void KMPM_initFaultCategories_UAF(ZMPM_IO* pOWN) {
	UMPM_voltage* cat1;
	UMPM_voltage* cat3;
	UMPM_voltage* cat4;
	UMPM_voltage* cat7;
	cat1 = (UMPM_voltage*) &pOWN->faultUAF.mr1;
	cat1->ib.uESpsm = 1;
	cat1->ib.eU_fuse = 1;
	cat1->ib.eU_MCB_VT = 1;
	cat3 = (UMPM_voltage*) &pOWN->faultUAF.mr3;
	cat3->ib.eU_1PhaseEarth = 1;
	cat4 = (UMPM_voltage*) &pOWN->faultUAF.mr4;
	cat4->ib.uStrangPSM = 1;
	cat4->ib.uShutOFFmin = 1;
	cat4->ib.uShutOFFmax = 1;
	cat7 = (UMPM_voltage*) &pOWN->faultUAF.mr7;
	cat7->ib.wShutOFFmin = 1;
	cat7->ib.wShutOFFmax = 1;
}

/*! \brief KMPM_initFaultCategories_IA
 * \return void
 * \param pOWN eigener Strukturzeiger
 *	
 * Fehlerkategorien zur IA
 */
static void KMPM_initFaultCategories_IA(ZMPM_IO* pOWN) {
	UMPM_current* cat6;
	UMPM_current* cat7;
	cat6 = (UMPM_current*) &pOWN->faultIA.mr6;
	cat7 = (UMPM_current*) &pOWN->faultIA.mr7;

	// momentan keine Einordnung

}

/*! \brief KMPM_initFaultCategories_FA
 * \return void
 * \param pOWN eigener Strukturzeiger
 *	
 * Fehlerkategorien zur FA
 */
static void KMPM_initFaultCategories_FA(ZMPM_IO* pOWN) {
	UMPM_Frequence* cat4;
	UMPM_Frequence* cat7;
	cat4 = (UMPM_Frequence*) &pOWN->faultFA.mr4;
	cat4->ib.eState = kMPM_uf_min + kMPM_uf_max;
	cat7 = (UMPM_Frequence*) &pOWN->faultFA.mr7;
	cat7->ib.eState = kMPM_wf_min + kMPM_wf_max;
}

/*! \brief KMPM_initFaultCategories_UBN
 * \return void
 * \param pOWN eigener Strukturzeiger
 *	
 * Fehlerkategorien zur UBN
 */
static void KMPM_initFaultCategories_UBN(ZMPM_IO* pOWN) {
	UMPM_voltage* cat3;
	UMPM_voltage* cat6;
	UMPM_voltage* cat7;
	cat3 = (UMPM_voltage*) &pOWN->faultUBN.mr3;
	cat3->ib.eUstate = kMPM_eU_Fault;
	cat6 = (UMPM_voltage*) &pOWN->faultUBN.mr6;
	cat6->ib.eU_fuse   = 1;
	cat6->ib.eU_MCB_VT = 1;
	cat7 = (UMPM_voltage*) &pOWN->faultUBN.mr7;
	cat7->ib.wShutOFFmin = 1;
	cat7->ib.wShutOFFmax = 1;
}


/*! \brief KMPM_initFaultCategories_UBF
 * \return void
 * \param pOWN eigener Strukturzeiger
 *	
 * Fehlerkategorien zur UBF
 */
static void KMPM_initFaultCategories_UBF(ZMPM_IO* pOWN) {
	UMPM_voltage* cat1;
	UMPM_voltage* cat2;
	UMPM_voltage* cat3;
	UMPM_voltage* cat5;
	UMPM_voltage* cat7;
	// R1-ESOF-Zuordnung
	cat1 = (UMPM_voltage*) &pOWN->faultUBF.mr1;
	cat1->ib.uESpsm = 1;
	cat1->ib.eU_fuse = 1;
	cat1->ib.eU_MCB_VT = 1;

	// R2-ESOF-Zuordnung
	cat2 = (UMPM_voltage*) &pOWN->faultUBF.mr2;
	cat2->ib.wShutOFFmin = 1;
	cat2->ib.uShutOFFmin = 1;

	// R3-SOF-Zuordnung
	cat3 = (UMPM_voltage*) &pOWN->faultUBF.mr3;
	cat3->ib.uShutOFFmax = 1;

	// R5-PSMa-Zuordnung
	cat5 = (UMPM_voltage*) &pOWN->faultUBF.mr5;
	cat5->ib.uNotSynchron = 1;
	cat5->ib.eUstate = kMPM_eU_Fault;

	// R7-Warnung-Zuordnung
	cat7 = (UMPM_voltage*) &pOWN->faultUBF.mr7;
	cat7->ib.wShutOFFmax = 1;
}

/*! \brief KMPM_initFaultCategories_IB
 * \return void
 * \param pOWN eigener Strukturzeiger
 *	
 * Fehlerkategorien zur IB
 */
static void KMPM_initFaultCategories_IB(ZMPM_IO* pOWN) {
	UMPM_current* cat6;
	UMPM_current* cat7;
	cat6 = (UMPM_current*) &pOWN->faultIB.mr6;
	cat7 = (UMPM_current*) &pOWN->faultIB.mr7;
	// momentan keine Einordnung
}

/*! \brief KMPM_initFaultCategories_FB
 * \return void
 * \param pOWN eigener Strukturzeiger
 *	
 * Fehlerkategorien zur FB
 */
static void KMPM_initFaultCategories_FB(ZMPM_IO* pOWN) {
	UMPM_Frequence* cat3;
	UMPM_Frequence* cat7;
	cat3 = (UMPM_Frequence*) &pOWN->faultFB.mr3;
	cat3->ib.eState = kMPM_uf_min + kMPM_uf_max;
	cat7 = (UMPM_Frequence*) &pOWN->faultFB.mr7;
	cat7->ib.eState = kMPM_wf_min + kMPM_wf_max;
}

/*! \brief KMPM_controlPLL_B
 * \anchor KMPM_controlPLL_B
 * \return void
 * \param pOWN eigener Strukturzeiger
 *	
 * Ansteuerung der B-PLL
 */
static void KMPM_controlPLL_B(ZMPM_IO* pOWN, ZSEQ_IO* pSEQ, ZREF_IO* pREF) {
	/*! Dem zu speisenden Netz (synchron/asynchron) entsprechend wird die B-PLL konfiguriert. \n
		*	mgliche Einstellungen sind \n
		* - 00: Sperren
		* - 01: PLL asynchr.Frequenz
		* - 10: PLL synchr. Frequenz
		* - 11: Halten
		* 
		* @dotfile dotPLLcontrol_B.txt
		*/
	int netType;
	netType = pREF->netType.actual;
	switch(netType)
	{
		case kNET_TYPE_ASYNC:
		{
			if(  pOWN->cB_yard.ib.eCBon  ||
			   ((pOWN->cB_yard.ib.cRequestUmess == kUNselect) && 
			     pOWN->cB_yard.ib.eUNallocated &&
				(pSEQ->wFeeding != kSEQ_FCmd_SAMb) && 
				(pSEQ->wFeeding != kSEQ_FCmd_LTb)  ) )
			{
				pOWN->statusPLL_B = ePLL_async;
				pOWN->cB_yard.ib.cSelectPLL = 1;
			}else
			{
				pOWN->statusPLL_B = ePLL_blocked;
				pOWN->cB_yard.ib.cSelectPLL = 0;
			}
			// Nur in asynchronen Netzen wird PLL zum ffnen des Schalters gehalten
			if((pSEQ->FMmode.checked != kFeeding_PSMb))
			{
				if(pOWN->Ein.cB_yard.ib.cSelectPLL == 3)
				{
					pOWN->statusPLL_B = ePLL_hold;
					pOWN->cB_yard.ib.cSelectPLL = 3;
				}
			}
		}break;
		case kNET_TYPE_SYNC:
		{
			pOWN->statusPLL_B = ePLL_sync;
			pOWN->cB_yard.ib.cSelectPLL = 2;
		}break;
		default:
			pOWN->cfgBoard.moduleFaults |= kMPM_noNetMode;
		break;
	}
	
	pOWN->msgPLL_B.faults = *(uint16*)&pOWN->statusPLL_B;
}

/*! \brief KMPM_controlPLL_A
 * \anchor KMPM_controlPLL_A
 * \return void
 * \param pOWN eigener Strukturzeiger
 *	
 * Ansteuerung der A-PLL
 */
static void KMPM_controlPLL_A(ZMPM_IO* pOWN, ZSEQ_IO* pSEQ, ZREF_IO* pREF) {
	/*! Dem Netz (syn/asyn) entsprechend wird die A-PLL konfiguriert. \n
	 *	mgliche Einstellungen : \n
	 * - 0: Sperren
	 * - 1: PLL asynchr.Frequenz
	 * - 2: PLL synchr. Frequenz
	 * - 3: Halten
	 *
	 * @dotfile dotPLLcontrol_A.txt
	 */
	eNET_TYPE	 netType;
	netType = pREF->netType.display;
	switch(netType)
	{
		case kNET_TYPE_ASYNC:
		{
			pOWN->statusPLL_A = ePLL_async;
			pOWN->cA_yard.ib.cSelectPLL = 1;
			// Halten, wenn A-Schalter geffnet wird
			if(pOWN->Ein.cA_yard.ib.cSelectPLL == 3)
			{
					pOWN->statusPLL_A = ePLL_hold;
					pOWN->cA_yard.ib.cSelectPLL = 3;
			}
		}break;
		case kNET_TYPE_SYNC:
		{
			pOWN->statusPLL_A = ePLL_sync;
			if(pREF->netType.released) 
			{
				pOWN->statusPLL_A = ePLL_sync;
				pOWN->cA_yard.ib.cSelectPLL = pOWN->wVarNet;
			}else{pOWN->cA_yard.ib.cSelectPLL = 2;}
		}break;
		default:
			pOWN->cfgBoard.moduleFaults |= kMPM_noNetMode;
		break;
	}
	pOWN->msgPLL_A.faults = *(uint16*)&pOWN->statusPLL_A;
}